/*
 * Twidere - Twitter client for Android
 *
 *  Copyright (C) 2012-2015 Mariotaku Lee <mariotaku.lee@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package twitter4j.internal.json;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.net.MalformedURLException;
import java.net.URL;

import twitter4j.TwitterException;
import twitter4j.URLEntity;

/**
 * A data class representing one single URL entity.
 * 
 * @author Mocel - mocel at guma.jp
 * @since Twitter4J 2.1.9
 */
/* package */final class URLEntityJSONImpl implements URLEntity {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1326410198426703277L;
	private int start = -1;
	private int end = -1;
	private URL url;
	private URL expandedURL;
	private String displayURL;

	/* For serialization purposes only. */
	/* package */URLEntityJSONImpl() {

	}

	/* package */URLEntityJSONImpl(final int start, final int end, final String url, final String expandedURL,
			final String displayURL) {
		super();
		this.start = start;
		this.end = end;
		try {
			this.url = new URL(url);
		} catch (final MalformedURLException e) {
			try {
				this.url = new URL("http://example.com/");
			} catch (final MalformedURLException ignore) {
			}
		}
		try {
			this.expandedURL = new URL(expandedURL);
		} catch (final MalformedURLException e) {
			try {
				this.expandedURL = new URL("http://example.com/");
			} catch (final MalformedURLException ignore) {
			}
		}
		this.displayURL = displayURL;
	}

	/* package */URLEntityJSONImpl(final JSONObject json) throws TwitterException {
		super();
		init(json);
	}

	@Override
	public boolean equals(final Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;

		final URLEntityJSONImpl that = (URLEntityJSONImpl) o;

		if (end != that.end) return false;
		if (start != that.start) return false;
		if (displayURL != null ? !displayURL.equals(that.displayURL) : that.displayURL != null) return false;
		if (expandedURL != null ? !expandedURL.toString().equalsIgnoreCase(that.expandedURL.toString())
				: that.expandedURL != null) return false;
		if (url != null ? !url.toString().equalsIgnoreCase(that.url.toString()) : that.url != null) return false;

		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getDisplayURL() {
		return displayURL;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getEnd() {
		return end;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public URL getExpandedURL() {
		return expandedURL;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public int getStart() {
		return start;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public URL getURL() {
		return url;
	}

	@Override
	public int hashCode() {
		int result = start;
		result = 31 * result + end;
		result = 31 * result + (url != null ? url.toString().hashCode() : 0);
		result = 31 * result + (expandedURL != null ? expandedURL.toString().hashCode() : 0);
		result = 31 * result + (displayURL != null ? displayURL.hashCode() : 0);
		return result;
	}

	@Override
	public String toString() {
		return "URLEntityJSONImpl{" + "start=" + start + ", end=" + end + ", url=" + url + ", expandedURL="
				+ expandedURL + ", displayURL=" + displayURL + '}';
	}

	private void init(final JSONObject json) throws TwitterException {
		try {
			final JSONArray indicesArray = json.getJSONArray("indices");
			start = indicesArray.getInt(0);
			end = indicesArray.getInt(1);

			try {
				url = new URL(json.getString("url"));
			} catch (final MalformedURLException ignore) {
			}

			if (!json.isNull("expanded_url")) {
				try {
					expandedURL = new URL(json.getString("expanded_url"));
				} catch (final MalformedURLException ignore) {
				}
			}
			if (!json.isNull("display_url")) {
				displayURL = json.getString("display_url");
			}
		} catch (final JSONException jsone) {
			throw new TwitterException(jsone);
		}
	}
}
